/****************************************************************************\
*  Name:           joytest.c                                                 *
*  Purpose:        Joystick / keyboard testing                               *
*  Author:         Yiannis Paschalidis (Alias "X") - alias@aleax.com         *
*  Last modified:  23/10/2000                                                *
\****************************************************************************/

#include <bios.h>
#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <stos.h>

//----------------------------------------------------------------------------
//Constants for joystick routines (STOSLib doesn't make them public)

#define JoyA_B1  0x10
#define JoyA_B2  0x20
#define JoyB_B1  0x40
#define JoyB_B2  0x80
#define JoyA_X   0x01
#define JoyA_Y   0x02
#define JoyB_X   0x04
#define JoyB_Y   0x08

//----------------------------------------------------------------------------
//Reads the raw values from the joystick port

void get_raw_stickpos(word *analog_x, word *analog_y, byte val1, byte val2)
{
   byte mask = val1 | val2;
   byte mask2 = val1;

   __asm__("   movw $0,%0
          \n   cli
          \n   movw $0x201,%%dx
          \n   outb %%al,%%dx
          \n 0:
          \n   incw %0
          \n   cmpw $0xFFFF,%0
          \n   je 3f
          \n   inb %%dx,%%al
          \n   andb %2,%%al
          \n   cmpb %2,%%al
          \n   je 0b
          \n   movw %0,%1 
          \n   andb %3,%%al
          \n   je 2f
          \n 1:
          \n   incw %0
          \n   cmpw $0xFFFF,%0
          \n   je 3f
          \n   inb %%dx,%%al
          \n   andb %2,%%al
          \n   jne 1b
          \n   jmp 3f
          \n 2:
          \n   incw %1
          \n   cmpw $0xFFFF,%1
          \n   je 3f
          \n   inb %%dx,%%al
          \n   andb %2,%%al
          \n   jne 2b
          \n 3:
          \n   sti
   "
          /*Outputs*/ : "=bx" (*analog_x), "=cx" (*analog_y)
          /*Inputs*/  : "g" (mask), "g" (mask2)
          /*Reg's*/   : "al","dx","memory"
   );  
}

void get_stickApos(word *analog_x, word *analog_y)
{
   get_raw_stickpos(analog_x, analog_y, JoyA_X, JoyA_Y);
}

void get_stickBpos(word *analog_x, word *analog_y)
{
   get_raw_stickpos(analog_x, analog_y, JoyB_X, JoyB_Y);
}

//----------------------------------------------------------------------------

void main(int argc, char *argv[])
{
   word stickAx, stickAy;
   word stickBx, stickBy;
   char keysdown[81];
   byte i,temp;

   //Initialise STOSLib
   initSTOS();

   //turn off keyboard chaining (we aren't using getch() etc).
   kbd_chain(OFF);

   //----------------
   //clear the screen
   //----------------

   textattr(0x0F);
   _setcursortype(_NOCURSOR);
   clrscr();

   //-------
   //borders
   //-------

   textattr(0x40);

   for (i=1 ; i<26 ; i++)
   {
      gotoxy(65, i);
      cprintf(" ");
   }

   gotoxy(65, 15);
   cprintf("                ");

   gotoxy(1, 24);
   cprintf(" Joy/Kbd tester 2000. Yiannis Paschalidis                       ");

   //-------------
   //Keymap header
   //-------------

   textattr(0x21);

   for (i=0 ; i<128 ; i++)
   {
      gotoxy(2+(i & 0xf)*4,(i/16)*3 + 1);
      cprintf("%x",i);
   }

   textattr(0x0F);

   //----------------
   //Keys down header
   //----------------

   gotoxy(1,25);
   cprintf("keys down: ");

   //------------
   //Joy A Header
   //------------

   gotoxy(70,1);
   cprintf("joy1:");
   gotoxy(67,3);
   cprintf("x-axis=");
   gotoxy(67,4);
   cprintf("y-axis=");
   gotoxy(67,5);
   cprintf("button1=");
   gotoxy(67,6);
   cprintf("button2=");

   //------------
   //Joy B Header
   //------------

   gotoxy(70,8);
   cprintf("joy2:");
   gotoxy(67,10);
   cprintf("x-axis=");
   gotoxy(67,11);
   cprintf("y-axis=");
   gotoxy(67,12);
   cprintf("button1=");
   gotoxy(67,13);
   cprintf("button2=");


   //---------------
   //6 button header
   //---------------

   gotoxy(67,16);
   cprintf("6 button stick");
   gotoxy(67,18);
   cprintf("x-axis=");
   gotoxy(67,19);
   cprintf("y-axis=");
   gotoxy(67,20);
   cprintf("button1=");
   gotoxy(67,21);
   cprintf("button2=");
   gotoxy(67,22);
   cprintf("button3=");
   gotoxy(67,23);
   cprintf("button4=");
   gotoxy(67,24);
   cprintf("button5=");
   gotoxy(67,25);
   cprintf("button6=");

   while(!kbd_keydown(SCAN_ESC))
   {
      keysdown[0]=0;

      //-----------
      //Keymap data
      //-----------

      for (i=0 ; i<128 ; i++)
      {
         gotoxy(2+(i & 0xf)*4,2+(i/16)*3);

         temp=kbd_keydown(i);

         cprintf("%d",temp);

         if (temp)
         {
            strcat(keysdown,key_names[i]);
            strcat(keysdown,",");
         }
      }

      //--------------
      //Keys down data
      //--------------

      gotoxy(12,25);
      cprintf("                                                  ");
      gotoxy(12,25);
      cprintf("%s",keysdown);

      get_stickApos(&stickAx, &stickAy);
      get_stickBpos(&stickBx, &stickBy);

      //----------
      //Joy A data
      //----------

      gotoxy(74,3);
      cprintf("%05d",stickAx);
      gotoxy(74,4);
      cprintf("%05d",stickAy);
      gotoxy(75,5);
      cprintf("%s",get_joybutton(JoyA_B1) ? "ON " : "OFF");
      gotoxy(75,6);
      cprintf("%s",get_joybutton(JoyA_B2) ? "ON " : "OFF");

      //----------
      //Joy B data
      //----------

      gotoxy(74,10);
      cprintf("%05d",stickBx);
      gotoxy(74,11);
      cprintf("%05d",stickBy);
      gotoxy(75,12);
      cprintf("%s",get_joybutton(JoyB_B1) ? "ON " : "OFF");
      gotoxy(75,13);
      cprintf("%s",get_joybutton(JoyB_B2) ? "ON " : "OFF");

      //-------------
      //6 button data
      //-------------

      gotoxy(74,18);
      cprintf("%05d",stickAx);
      gotoxy(74,19);
      cprintf("%05d",stickAy);
      gotoxy(75,20);
      cprintf("%s",get_joybutton(JoyA_B1) ? "ON " : "OFF");
      gotoxy(75,21);
      cprintf("%s",get_joybutton(JoyA_B2) ? "ON " : "OFF");
      gotoxy(75,22);
      cprintf("%s",get_joybutton(JoyB_B1) ? "ON " : "OFF");
      gotoxy(75,23);
      cprintf("%s",get_joybutton(JoyB_B2) ? "ON " : "OFF");
      gotoxy(75,24);
      cprintf("%s",stickBx<50 ? "ON " : "OFF");
      gotoxy(75,25);
      cprintf("%s",stickBy<50 ? "ON " : "OFF");
   }

   //exit cleanly
   _setcursortype(_NORMALCURSOR);
   kbd_chain(ON);
   closeSTOS();
   exit(0);
}
